@@ -1,4 +1,4 @@
-# Copyright Louis Paternault 2017-2021
+# Copyright Louis Paternault 2017-2022
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as
@@ -26,12 +26,11 @@ import subprocess
 import tempfile
 import threading
 
-from evariste import errors
-from evariste.plugins.action import Action, Report
-from evariste.utils import yesno
-import evariste
+from ... import errors
+from ...utils import yesno
+from . import Action, Report
 
-LOGGER = logging.getLogger(evariste.__name__)
+LOGGER = logging.getLogger("evariste")
 STRACE_RE = re.compile(
     r'^\d* *open(at)?\((.*, )?"(?P<name>.*)",.*O_RDONLY.*\) = *[^ -].*'
 )
@@ -93,7 +92,7 @@ def system_strace(command, path, log, depends):
                 match.groupdict()["name"]
             )
             if name.resolve() != path.from_fs.resolve():  # pylint: disable=no-member
-                if path.vcs.is_versionned(name):
+                if name in path.vcs:
                     depends.add(name)
 
     def _process_strace(pipe):
