@@ -1,4 +1,4 @@
-# Copyright Louis Paternault 2015-2021
+# Copyright Louis Paternault 2015-2022
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as
@@ -15,48 +15,50 @@
 
 """Plugin to copy files at the end of compilation and rendering."""
 
+import contextlib
 import glob
+import logging
+import shutil
 
-from evariste import plugins, utils
-from evariste import hooks
+from ..hooks import contexthook
+from . import Plugin
+
+LOGGER = logging.getLogger("evariste.plugins.copy")
 
 
-class _Close(hooks.ContextHook):
-    """Hook to copy files at the end of compilation."""
+class Copy(Plugin):
+    """Copy files at the end of compilation."""
 
     # pylint: disable=too-few-public-methods
 
-    hookname = "Builder.close"
+    keyword = "copy"
 
-    def __init__(self, plugin, builder):
+    @contexthook("Builder.close")
+    @contextlib.contextmanager
+    def copy(self, builder):
+        """Copy files (as configured in the configuration file) at the end of compilation."""
         # pylint: disable=unused-argument
-        super().__init__(plugin)
 
-    def __exit__(self, *args, **kwargs):
+        yield
+
         for key in self.local.setup:
             if key.startswith("copy"):
                 if isinstance(self.local.setup[key], str):
                     arguments = self.local.setup[key].split()
                 else:
                     arguments = self.local.setup[key]
+                if len(arguments) <= 1:
+                    raise ValueError(
+                        f"""[copy] Option "{key}" should be "SOURCE [SOURCE] DEST"."""
+                    )
                 try:
-                    sources, dest = arguments
+                    sources = arguments[0:-1]
+                    dest = arguments[-1]
                 except (ValueError, TypeError) as error:
                     raise ValueError(
-                        f"""[copy] Option "{key}" should be "SOURCE DEST"."""
+                        f"""[copy] Option "{key}" should be "SOURCE [SOURCE] DEST"."""
                     ) from error
-                if dest is None:
-                    continue
-                for path in glob.iglob(sources):
-                    utils.copytree(path, dest)
-        return super().__exit__(*args, **kwargs)
-
-
-class Copy(plugins.PluginBase):
-    """Copy files at the end of compilation."""
-
-    # pylint: disable=too-few-public-methods
-
-    plugin_type = ""
-    keyword = "copy"
-    hooks = [_Close]
+                LOGGER.info(f"""Copying '{" ".join(sources)}' to '{dest}'.""")
+                for source in sources:
+                    for path in glob.iglob(source):
+                        shutil.copytree(path, dest, dirs_exist_ok=True)
