@@ -1,4 +1,4 @@
-# Copyright Louis Paternault 2015-2020
+# Copyright Louis Paternault 2015-2022
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as
@@ -13,13 +13,30 @@
 # You should have received a copy of the GNU Affero General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-"""Default HTML file renderer"""
+"""Default HTML file renderer
 
-from evariste.plugins.renderer.jinja2.file import Jinja2FileRenderer
+.. autoclass:: HtmlFileRenderer
+   :members:
+
+"""
+
+from ...jinja2.file import Jinja2FileRenderer
 
 
 class HtmlFileRenderer(Jinja2FileRenderer):
-    """Default HTML file renderer"""
+    """Default HTML file renderer.
+
+    This displays the file name together with the file source.
+
+    To write another file renderer, you can:
+
+    - define a new
+      :attr:`~evariste.plugins.renderer.jinja2.file.Jinja2FileRenderer.template` to use:
+    - overwrite the default
+      :meth:`~evariste.plugins.renderer.jinja2.file.Jinja2FileRenderer.match` method;
+    - overwrite the default
+      :meth:`~evariste.plugins.renderer.jinja2.file.Jinja2FileRenderer.render` method.
+    """
 
     # pylint: disable=too-few-public-methods
 
