@@ -1,4 +1,4 @@
-# Copyright Louis Paternault 2017-2021
+# Copyright Louis Paternault 2017-2022
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -13,11 +13,11 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-"""IkiWiki markdown README renderer"""
+"""Markdown README renderer"""
 
 import markdown
 
-from evariste.plugins.renderer.html.readme import HtmlReadmeRenderer
+from . import HtmlReadmeRenderer
 
 
 class MdwnReadmeRenderer(HtmlReadmeRenderer):
@@ -26,7 +26,7 @@ class MdwnReadmeRenderer(HtmlReadmeRenderer):
     keyword = "renderer.html.readme.mdwn"
     extensions = ["mdwn", "md"]
 
-    def render(self, path, context):
+    def render(self, tree):
         """Render file as html code."""
-        with open(path, encoding="utf8") as source:
+        with open(tree.from_fs, encoding="utf8") as source:
             return markdown.markdown(source.read())
