@@ -1,4 +1,4 @@
-# Copyright Louis Paternault 2016-2021
+# Copyright Louis Paternault 2016-2022
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as
@@ -16,16 +16,17 @@
 
 """Text renderer: print report on terminal."""
 
+import contextlib
 import logging
 import sys
 
 import unidecode
 
-from evariste.plugins.renderer import Renderer
-from evariste.utils import yesno
-import evariste
+from ...hooks import contexthook
+from ...utils import yesno
+from .. import Plugin
 
-LOGGER = logging.getLogger(evariste.__name__)
+LOGGER = logging.getLogger("evariste")
 
 
 class TreePrettyPrinter:
@@ -92,20 +93,22 @@ class TreePrettyPrinter:
         if not self.reverse:
             self.print_function(" ".join(line))
 
-        if last:
-            postfix = "  "
-        else:
-            postfix = "│ "
+        if tree.is_dir():
 
-        subtree = [branch for branch in tree.values() if self.display(branch)]
-        if self.reverse:
-            lastcounter = 1
-        else:
-            lastcounter = len(subtree)
-        counter = 0
-        for sub in sorted(subtree, reverse=self.reverse):
-            counter += 1
-            self.pprint(sub, prefix=prefix + postfix, last=(counter == lastcounter))
+            if last:
+                postfix = "  "
+            else:
+                postfix = "│ "
+
+            subtree = [branch for branch in tree.values() if self.display(branch)]
+            if self.reverse:
+                lastcounter = 1
+            else:
+                lastcounter = len(subtree)
+            counter = 0
+            for sub in sorted(subtree, reverse=self.reverse):
+                counter += 1
+                self.pprint(sub, prefix=prefix + postfix, last=(counter == lastcounter))
 
         if self.reverse:
             self.print_function(" ".join(line))
@@ -114,7 +117,7 @@ class TreePrettyPrinter:
 ALLOWED_DISPLAY_OPTIONS = ["errors", "all", "errors_or_all"]
 
 
-class TextRenderer(Renderer):
+class TextRenderer(Plugin):
     """Text renderer: print report on terminal."""
 
     keyword = "renderer.text"
@@ -125,7 +128,12 @@ class TextRenderer(Renderer):
         "display": "all",
     }
 
-    def render(self, tree):
+    @contexthook("Builder.compile")
+    @contextlib.contextmanager
+    def render(self, builder):
+        """Print the report to the standard output."""
+
+        yield
 
         # Option 'ascii'
         if yesno(self.local.setup["ascii"]):
@@ -155,7 +163,7 @@ class TextRenderer(Renderer):
             )
             displayoption = self.default_setup["display"]
         if displayoption == "errors_or_all":
-            if tree.report.success:
+            if builder.tree.report.success:
                 displayoption = "all"
             else:
                 displayoption = "errors"
@@ -177,4 +185,4 @@ class TextRenderer(Renderer):
             print_function=print_function,
             reverse=reverse,
             display=display,
-        ).pprint(tree)
+        ).pprint(builder.tree)
