@@ -1,4 +1,4 @@
-# Copyright Louis Paternault 2016
+# Copyright Louis Paternault 2016-2022
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as
@@ -17,7 +17,7 @@
 
 import os
 
-from evariste.plugins.vcs import VCS
+from . import VCS
 
 
 class FS(VCS):
@@ -34,7 +34,7 @@ class FS(VCS):
             for filename in filenames:
                 yield os.path.relpath(os.path.join(dirpath, filename), self.workdir)
 
-    def is_versionned(self, path):
+    def __contains__(self, path):
         return path.is_file() and (self.workdir in path.parents)
 
     @property
