@@ -1,4 +1,4 @@
-# Copyright Louis Paternault 2015-2021
+# Copyright Louis Paternault 2015-2022
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as
@@ -13,11 +13,18 @@
 # You should have received a copy of the GNU Affero General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-"""Share global data between evariste objects."""
+"""Share global data between evariste objects.
+
+More information in :ref:`shared`.
+
+.. autoclass:: Shared
+   :members:
+
+"""
 
 from enum import Enum
 
-from evariste.utils import DeepDict
+from .utils import DeepDict
 
 
 class ViewPoint(Enum):
@@ -172,10 +179,36 @@ class Shared:
             else:
                 setattr(self, attr, SharedDeepDict.from_attr(attr))
 
-    def get_plugin_view(self, keyword):
-        """Get this data, from the point of view of a plugin."""
+    def get_plugin_view(self, keyword: str) -> _SharedView:
+        """Get this data, from the point of view of a plugin.
+
+        Let's define a ``shared`` object, and its "plugin view":
+
+        .. code-block:: python
+
+           shared = Shared(...)
+           view = self.get_plugin_view(foo)
+
+        Now, when both getting and setting data:
+
+        - ``view.plugin`` is equivalent to ``shared.plugin[foo]``;
+        - ``view.tree[bar]`` is equivalent to ``shared.tree[bar][foo]``;
+        - ``view.setup`` is equivalent to ``shared.setup[foo]``.
+        """
         return _SharedView(ViewPoint.plugin, keyword, self)
 
-    def get_tree_view(self, path):
-        """Get this data, from the point of view of a tree."""
+    def get_tree_view(self, path: str) -> _SharedView:
+        """Get this data, from the point of view of a tree.
+
+        Let's define a ``shared`` object, and its "plugin view":
+
+        .. code-block:: python
+
+           shared = Shared(...)
+           view = self.get_tree_view(foo)
+
+        Now, when both getting and setting data:
+
+        - ``view.tree[bar]`` is equivalent to ``shared.tree[foo][bar]``.
+        """
         return _SharedView(ViewPoint.tree, path, self)
