@@ -1,4 +1,4 @@
-# Copyright Louis Paternault 2016-2021
+# Copyright Louis Paternault 2016-2022
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as
@@ -13,9 +13,48 @@
 # You should have received a copy of the GNU Affero General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-"""Directory representation and compilation."""
+"""Directory representation and compilation.
 
-from concurrent.futures import ThreadPoolExecutor
+A :class:`Tree` is an abstract class representing a directory structure
+(a directory with files and nested directories).
+Its implementations are:
+
+    - :class:`File`: a file;
+    - :class:`Directory`: a directory;
+    - :class:`Root`: the root directory being processed.
+
+:class:`Tree`
+-------------
+
+.. autoclass:: Tree
+   :members:
+
+:class:`File`
+-------------
+
+.. autoclass:: File
+   :members:
+
+:class:`Directory`
+------------------
+
+.. autoclass:: Directory
+   :members:
+   :special-members: __iter__, __contains__, __getitem__, __delitem__
+
+:class:`Root`
+-------------
+
+.. autoclass:: Root
+   :members:
+
+"""
+
+# Can be removed starting with python3.11
+from __future__ import annotations
+
+import abc
+import datetime
 import functools
 import itertools
 import logging
@@ -24,16 +63,21 @@ import pathlib
 import shlex
 import tarfile
 import traceback
+import typing
+from concurrent.futures import ThreadPoolExecutor
 
-from evariste import plugins, utils, errors
-from evariste.plugins.action import DirectoryAction
-from evariste.hooks import methodhook
+from . import errors, plugins, utils
+from .hooks import setmethodhook
+from .plugins.action import DirectoryAction
+
+if typing.TYPE_CHECKING:
+    from .shared import Shared
 
 LOGGER = logging.getLogger(__name__)
 
 
 def get_tree_plugins(self, *args, **kwargs):
-    """Given the argument of :method:`Tree.__init__`, return a :class:`PluginLoader` instance."""
+    """Given the argument of :method:`Tree.__init__`, return a :class:`Loader` instance."""
     # pylint: disable=unused-argument
     if isinstance(self, Root):
         root = self
@@ -45,31 +89,60 @@ def get_tree_plugins(self, *args, **kwargs):
 
 
 @functools.total_ordering
-class Tree:
-    """Directory tree"""
+class Tree(metaclass=abc.ABCMeta):
+    """A file system tree.
+
+    A directory, that contains files and has subdirectories.
+
+    :param pathlib.Path path: Relative path (relative to the root of this tree).
+    :param typing.Optional[Directory] parent: Directory containing this file or directory.
+
+    """
 
     # pylint: disable=too-many-instance-attributes
 
-    @methodhook(getter=get_tree_plugins)
-    def __init__(self, path, *, parent):
-        self._subpath = {}
-        self.report = None
-        self.parent = parent
-        self.config = None
+    @setmethodhook(getter=get_tree_plugins)
+    def __init__(
+        self, path: pathlib.Path, *, parent: typing.Optional[Directory] = None
+    ):
+        #: Once the file has been :meth:`compiled <File.compile>`,
+        #: the report (compilation log, if any) is saved here.
+        self.report: typing.Union[plugins.action.Report, None] = None
+
+        #: Parent directory (copied from constructor argument).
+        self.parent: typing.Union[Tree, None] = parent
+
+        #: Computed configuration for this file. See :ref:`evsconfig`.
+        #: Note: This attribute is :class:`None` until :meth:`Tree.set_config` has been called.
+        self.config: typing.Union[utils.DeepDict, None] = None
+
         if parent is not None:
-            self.basename = pathlib.Path(path)
-            self.from_fs = parent.from_fs / self.basename
-            self.from_source = parent.from_source / self.basename
-            self.vcs = self.parent.vcs
-            self.shared = self.parent.shared
+            #: Name of the tree (path, relative to its :attr:`Tree.parent`).
+            self.basename: pathlib.Path = pathlib.Path(path)
+
+            #: Absolute path
+            self.from_fs: pathlib.Path = parent.from_fs / self.basename
+
+            #: Path, relative to the :class:`Root`.
+            self.from_source: pathlib.Path = parent.from_source / self.basename
+
+            #: VCS plugin
+            self.vcs: plugins.vcs.VCS = self.parent.vcs
+
+            #: Common data shared with every :class:`Tree` and :class:`~evariste.plugins.Plugin`
+            #: of this :class:`~evariste.builder.Builder`.
+            self.shared: Shared = self.parent.shared
+
+        #: Same as :attr:`Tree.shared`, but from a tree point of view:
+        #: see :meth:`~evariste.shared.Shared.get_tree_view`.
         self.local = self.shared.get_tree_view(path)
 
     def __hash__(self):
         return hash((hash(self.parent), self.from_fs))
 
     @property
-    def relativename(self):
-        """Return a 'relative' path.
+    def relativename(self) -> pathlib.Path:
+        """Return a *relative* name.
 
         * For root, return path relative to file system (or directory of setup file).
         * For non-root, return path relative to parent (i.e. basename of path).
@@ -84,7 +157,7 @@ class Tree:
         return False
 
     @property
-    def depth(self):
+    def depth(self) -> int:
         """Return the depth of the path.
 
         The root has depth 0, and depth of each path is one more than the depth
@@ -95,22 +168,25 @@ class Tree:
         return 1 + self.parent.depth
 
     @property
-    def root(self):
+    def root(self) -> Root:
         """Return the root of the tree."""
         if self.is_root():
             return self
         return self.parent.root
 
-    def find(self, path):
+    def find(
+        self, path: typing.Union[str, pathlib.Path, typing.Tuple[str]]
+    ) -> typing.Union[Tree, False]:
         """Return the tree object corresponding to ``path`` if it exists; False otherwise.
 
         Argument can be:
-        - a string (:type:`str`);
-        - a :type:`pathlib.Path` object;
+
+        - a string (:class:`str`);
+        - a :class:`pathlib.Path` object;
         - a tuple of strings, as a list of directories and (optional) final file.
         """
         if isinstance(path, str):
-            pathtuple = pathlib.Path(path)
+            pathtuple = [path]
         elif isinstance(path, pathlib.Path):
             pathtuple = path.parts
         else:
@@ -122,54 +198,9 @@ class Tree:
             return self[pathtuple[0]].find(pathtuple[1:])
         return False
 
-    def add_subpath(self, sub):
-        """Add a subpath"""
-        if len(sub.parts) > 1:
-            self[sub.parts[0]].add_subpath(pathlib.Path(*sub.parts[1:]))
-        else:
-            self[sub.parts[0]]  # pylint: disable=pointless-statement
-
-    def __iter__(self):
-        return iter(self._subpath)
-
-    def keys(self):
-        """Iterator over subpaths (as :class:`str` objects)."""
-        yield from self._subpath.keys()
-
-    def values(self):
-        """Iterator over subpaths (as :class:`Tree` objects)."""
-        yield from self._subpath.values()
-
-    def __contains__(self, key):
-        return key in self._subpath
-
-    def __getitem__(self, key):
-        if key not in self:
-            if (self.from_fs / key).is_dir():
-                path_type = Directory
-            else:
-                path_type = File
-            self._subpath[key] = path_type(key, parent=self)
-        return self._subpath[key]
-
-    def __delitem__(self, item):
-        pathitem = pathlib.Path(item)
-        if len(pathitem.parts) != 1:
-            raise errors.EvaristeBug(
-                "Argument '{}' should be a single directory or file, "
-                "not a 'directory/directory' or directory/file'."
-            )
-        if str(pathitem) in self:
-            del self._subpath[str(pathitem)]
-            if (not self._subpath) and self.is_dir() and self.parent is not None:
-                del self.parent[self.basename]
-
     def __str__(self):
         return self.from_fs.as_posix()
 
-    def __len__(self):
-        return len(self._subpath)
-
     def __eq__(self, other):
         return self.from_source == other.from_source
 
@@ -180,35 +211,29 @@ class Tree:
             return False
         return self.from_source < other.from_source
 
-    def is_dir(self):
+    def is_dir(self) -> bool:
         """Return `True` iff `self` is a directory."""
         return issubclass(self.__class__, Directory)
 
-    def is_file(self):
+    def is_file(self) -> bool:
         """Return `True` iff `self` is a file."""
         return issubclass(self.__class__, File)
 
-    def walk(self, dirs=False, files=True):
-        """Iterator over files or directories  of `self`.
-
-        :param bool dirs: If `False`, do not yield directories.
-        :param bool files: If `False`, do not yield files.
-
-        Directories are yielded *before* subfiles and subdirectories.
-        """
-        if (dirs and self.is_dir()) or (files and self.is_file()):
+    def walk(self, dirs: bool = False, files: bool = True) -> typing.Iterable[Tree]:
+        """Iterator over itself."""
+        #  pylint: disable=unused-argument
+        if files and self.is_file():
             yield self
-        for sub in sorted(self):
-            yield from self[sub].walk(dirs, files)
 
-    def count(self, dirs=False, files=True):
+    def count(self, dirs: bool = False, files: bool = True) -> int:
         """Count the number of files or directories in this tree."""
         return sum(1 for _ in self.walk(dirs, files))
 
-    def prune(self, path):
+    def prune(self, path: typing.Union[pathlib.Path, str, typing.Tuple[str]]):
         """Remove a file.
 
         Argument can be either:
+
         - a :class:`pathlib.Path`,
         - a :class:`tuple`,
         - or a :class:`str` (which would be converted to a :class:`pathlib.Path`.
@@ -234,8 +259,21 @@ class Tree:
         else:
             self[parts[0]].prune(parts[1:])
 
-    def format(self, string):
-        """Format given string, with several variables related to ``self``."""
+    def format(self, string: str) -> str:
+        """Format given string, with several variables related to ``self``.
+
+        Here are the replacements (with example :file:`/home/louis/repo/foo/bar.txt`):
+
+        - ``{dirname}`` (:file:`/home/louis/repo/foo`): the name of the directory.
+          Note that most of the time, this is useless, since when compiling a file,
+          the working directory is the directory of this file (i.e. ``{dirname}``).
+        - ``{filename}`` (:file:`bar.txt`): The file name (without directory).
+        - ``{fullname}`` (:file:`/home/louis/repo/foo/bar.txt`): The file name (with directory).
+        - ``{extension}`` (:file:`txt`): The extension (without the dot).
+          If the file has several extensions (e.g. :file:`foo.tar.gz`),
+          this is only the last one ``gz``.
+        - ``{basename}`` (:file:`bar`): The file name, without directory and extension.
+        """
         suffix = self.from_source.suffix
         if suffix.startswith("."):
             suffix = suffix[1:]
@@ -247,75 +285,99 @@ class Tree:
             basename=self.basename.stem,
         )
 
-    def iter_readmes(self, renderers):
-        """Iterate over readme files."""
-        for renderer in renderers:
-            for filename in renderer.iter_readme(self.from_fs, directory=self.is_dir()):
-                yield pathlib.Path(filename).relative_to(self.root.from_fs)
-                return
-
-    def full_depends(self):
-        """Return the list of all dependencies of this tree (recursively for directories)."""
+    @abc.abstractmethod
+    def full_depends(self) -> typing.Iterable[pathlib.Path]:
+        """Iterate over all dependencies of this tree (recursively for directories)."""
         raise NotImplementedError()
 
 
 class Directory(Tree):
     """Directory"""
 
-    def prune_ignored(self):
-        """Prune ignored files."""
-        ignorename = self.from_fs / ".evsignore"
-        if not ignorename.exists():
-            return
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
 
-        self.prune(".evsignore")
+        self._subpath = {}
 
-        with open(ignorename, encoding="utf8") as file:
-            for line in file.readlines():
-                line = line.strip()
-                if not line:
-                    continue
-                if line.startswith("#"):
-                    continue
-                try:
-                    # Fake glob, to catch lines with invalid syntax.
-                    for _ in self.from_fs.glob(line):
-                        break
-                except ValueError as error:
-                    LOGGER.warning(
-                        f"Line '{line}' has been ignored in file '{ignorename}': {error}."
-                    )
-                    continue
-                for filename in self.from_fs.glob(line):
-                    self.prune(filename.relative_to(self.from_fs))
+    def add_subpath(self, sub: typing.List[pathlib.Path]):
+        """Add a path to the tree (relative to ``self``)."""
+        if len(sub.parts) > 1:
+            self[sub.parts[0]].add_subpath(pathlib.Path(*sub.parts[1:]))
+        else:
+            self[sub.parts[0]]  # pylint: disable=pointless-statement
+
+    def __iter__(self) -> typing.Iterable[str]:
+        """Iterate over subpaths (this function is not recursive)."""
+        return iter(self._subpath)
+
+    def keys(self) -> typing.Iterable[str]:
+        """Iterator over subpaths (as :class:`str` objects)."""
+        yield from self._subpath.keys()
+
+    def values(self) -> typing.Iterable[Tree]:
+        """Iterator over subpaths (as :class:`Tree` objects)."""
+        yield from self._subpath.values()
+
+    def __contains__(self, key: str) -> bool:
+        """Return ``True`` if key (a single file name or directory) is in this directory."""
+        return key in self._subpath
+
+    def __getitem__(self, key: str) -> Tree:
+        """Return subfile or subdirectory ``self.from_fs / key``.
+
+        If it does not exist, it is created first.
+        """
+        if key not in self:
+            if (self.from_fs / key).is_dir():
+                path_type = Directory
+            else:
+                path_type = File
+            self._subpath[key] = path_type(key, parent=self)
+        return self._subpath[key]
+
+    def __delitem__(self, item: str):
+        """Remove a subfile or subdirectory.
+
+        If, after deletion, ``self`` is an empty directory (and is not root),
+        ``self`` is remove from its parent.
+        """
+        pathitem = pathlib.Path(item)
+        if len(pathitem.parts) != 1:
+            raise errors.EvaristeBug(
+                "Argument '{}' should be a single directory or file, "
+                "not a 'directory/directory' or directory/file'."
+            )
+        if str(pathitem) in self:
+            del self._subpath[str(pathitem)]
+            if (not self._subpath) and self.is_dir() and self.parent is not None:
+                del self.parent[self.basename]
+
+    def walk(self, dirs: bool = False, files: bool = True) -> typing.Iterable[Tree]:
+        """Iterator over files or directories  of ``self``.
+
+        :param bool dirs: If `False`, do not yield directories.
+        :param bool files: If `False`, do not yield files.
+
+        Directories are yielded *before* subfiles and subdirectories.
+        """
+        if dirs and self.is_dir():
+            yield self
+        for sub in sorted(self):
+            yield from self[sub].walk(dirs, files)
 
     def compile(self):
         """Compile directory."""
         self.report = DirectoryAction(self.shared).compile(self)
 
-    def iter_readmes(self, renderers):
-        renderers = list(renderers)
-        yield from super().iter_readmes(renderers)
-
-        for sub in self:
-            yield from self[sub].iter_readmes(renderers)
-
-    def full_depends(self):
+    def full_depends(self) -> typing.Iterable[pathlib.Path]:
         for sub in self:
             yield from self[sub].full_depends()
 
 
 class File(Tree):
-    """File"""
+    """A file"""
 
-    def prune_ignored(self):
-        """Prune ignored files"""
-        for name in ["{}.evsignore", ".{}.evsignore"]:
-            if (self.parent.from_fs / name.format(self.basename)).exists():
-                self.parent.prune(name.format(self.basename))
-                self.parent.prune(self.basename)
-
-    @methodhook()
+    @setmethodhook()
     def compile(self):
         """Compile file."""
         # pylint: disable=too-many-branches
@@ -358,19 +420,15 @@ class File(Tree):
         ):
             for name in self.parent.from_fs.glob(regexp):
                 if name.resolve() != self.from_fs.resolve():
-                    if self.vcs.is_versionned(name):
+                    if name in self.vcs:
                         self.report.depends.add(name)
 
-    def iter_depends(self):
-        """Iterate over dependencies."""
-        for path in self.report.depends:
-            yield path.resolve().relative_to(self.root.from_fs.resolve())
-
-    @methodhook()
-    def make_archive(self, destdir):
+    @setmethodhook()
+    def make_archive(self, destdir: pathlib.Path) -> pathlib.Path:
         """Make an archive of ``self`` and its dependency.
 
-        Stes are:
+        Steps are:
+
         - build the archive;
         - copy it to ``destdir``;
         - return the path of the archive, relative to ``destdir``.
@@ -428,114 +486,104 @@ class File(Tree):
                 )
         return archivepath
 
-    def last_modified(self):
+    def last_modified(self) -> datetime.datetime:
         """Return the last modified date and time of ``self``."""
         return self.vcs.last_modified(self.from_fs)
 
-    def full_depends(self):
+    def full_depends(self) -> typing.Iterable[pathlib.Path]:
         yield from self.report.full_depends
 
-    def depends(self):
+    def depends(self) -> typing.Iterable[pathlib.Path]:
         """Iterator over dependencies of this file (but not the file itself)."""
         yield from self.report.depends
 
 
 class Root(Directory):
-    """Root object (directory with no parents).
-
-    Note that before calling :method:`Root.root_compile`,
-    one must call :method:`Root.finalize`
-    (it prune trees of ignored files, match files with their configuration…).
-    This is automatically done if this object is created as a context manager, at when exiting it::
-
-        with Root("foo") as root:
-            root.add_subpath("bar")
-        root.root_compile()
-
-    This is also automatically done when object is created using factory :method:`Root.from_vcs`.
-    """
+    """Root object (directory with no parents)."""
 
     def __init__(self, path, *, vcs=None, shared=None):
         self.vcs = vcs
         self.shared = shared
-        self.from_fs = path
+        self.from_fs = pathlib.Path.cwd() / path
         self.from_source = pathlib.Path(".")
         self.basename = pathlib.Path(".")
-        self._finalized = False
-        super().__init__(path, parent=None)
+        super().__init__(path)
 
-    def __enter__(self):
-        return self
+    @staticmethod
+    def _config2file(config):
+        """Given a config (with a name ending in '.evsconfig', return the corresponding tree.
 
-    def __exit__(self, *args, **kwargs):
-        self.finalize()
+        If no such file is found, raise ValueError.
+        """
+        if str(config.basename) == ".evsconfig":
+            # Directory
+            return config.parent
 
-    def finalize(self):
-        """Remove ignored files, and compute configuration."""
-        # Remove ignored files
-        for tree in list(self.walk(dirs=True, files=True)):
-            tree.prune_ignored()
+        # File
+        # First try: `config` is SOMETHING.evsconfig and SOMETHING is in the tree
+        if tree := config.parent.find(str(config.basename)[: -len(".evsignore")]):
+            return tree
+        # Second try: `config` is .SOMETHING.evsconfig and SOMETHING is in the tree
+        if tree := config.parent.find(str(config.basename)[1 : -len(".evsignore")]):
+            return tree
 
-        # Set and remove configuration files
-        self.set_config()
-
-        self._finalized = True
+        raise ValueError()
 
     def _find_config(self):
-        # Finding (potential) configuration files for directories
-        for tree in self.walk(dirs=True, files=False):
-            config = tree.from_fs / ".evsconfig"
-            if config.exists():
-                yield tree.from_source, config, utils.read_config(
-                    config, allow_no_value=True
-                )
-
-        # Finding (potential) configuration files for files
-        for tree in self.walk(dirs=False, files=True):
-            for config in [".{}.evsconfig", "{}.evsconfig"]:
-                config = tree.parent.from_fs / config.format(tree.basename)
-                if config.exists():
-                    yield tree.from_source, config, utils.read_config(
-                        config, allow_no_value=True
-                    )
+        """Iterate over configuration files for directories"""
+        for config in self.walk():
+            if str(config).endswith("evsconfig"):
+                content = utils.read_config(config.from_fs, allow_no_value=True)
+                source = content.get("setup", "source", fallback=None)
+                if source is None:
+                    try:
+                        tree = self._config2file(config)
+                    except ValueError:
+                        logging.warning(
+                            f"Found a configuration file '{config}' with no related file. Processing it as a normal file."  # pylint: disable=line-too-long
+                        )
+                        continue
+                else:
+                    tree = self.find((config.parent.from_source / source))
+                    if not tree:
+                        logging.warning(
+                            f"Configuration file '{config}' refer to a non-existent source '{source}'."  # pylint: disable=line-too-long
+                        )
+                        continue
+                yield tree, config, content
 
     def set_config(self):
-        """Compute the configuration of each file of the thee.
+        """Compute the configuration of each file of the tree.
 
         That is:
-        look for the file that configure it
-        (typically `foo.evsconfig` is the configuration for file `foo`),
-        load it,
-        and complete it using the recursive configuration of parent directories.
+
+        - look for the file that configure it
+          (typically ``foo.evsconfig`` is the configuration for file ``foo``),
+        - load it,
+        - and complete it using the recursive configuration of parent directories.
         """
         recursive = {}
 
         # Assign configuration to files
-        for tree, configname, config in list(self._find_config()):
-            if utils.yesno(config.get("setup", "recursive", fallback=False)):
-                recursive[tree] = config
-                self.prune(configname.relative_to(self.from_fs))
+        for tree, configname, configcontent in list(self._find_config()):
+            if utils.yesno(configcontent.get("setup", "recursive", fallback=False)):
+                recursive[tree] = configcontent
+                self.prune(configname.from_source)
                 continue
 
-            if config.get("setup", "source", fallback=False):
-                if tree.is_dir():
-                    tree = tree / config["setup"]["source"]
-                else:
-                    tree = tree.parent / config["setup"]["source"]
-
-            tree = self.find(tree)
+            tree = self.find(tree.from_source)
             if tree.config is not None:
                 LOGGER.warning(  # pylint: disable=logging-fstring-interpolation
                     f"""Configuration file "{configname}" has been ignored for "{tree.from_fs}": another configuration file already applies."""  # pylint: disable=line-too-long
                 )
                 continue
 
-            self.prune(configname.relative_to(self.from_fs))
-            tree.config = utils.DeepDict.from_configparser(config)
+            self.prune(configname.from_source)
+            tree.config = utils.DeepDict.from_configparser(configcontent)
 
         # Propagate recursive configuration from directories
         for directory, config in sorted(recursive.items(), reverse=True):
-            for tree in self.find(directory).walk(dirs=True, files=True):
+            for tree in self.find(directory.from_source).walk(dirs=True, files=True):
                 if tree.config is None:
                     tree.config = utils.DeepDict.from_configparser(config)
                 else:
@@ -547,15 +595,22 @@ class Root(Directory):
                 tree.config = utils.DeepDict(2)
 
     @staticmethod
-    def is_root():
+    def is_root() -> bool:
         return True
 
     def root_compile(self):
         """Recursively compile files.."""
-        if not self._finalized:
-            raise Exception(
-                "You must call method `finalize()` before calling `root_compile()`."
+        # Set and remove configuration files
+        self.set_config()
+
+        # Prune files that are no longer needed.
+        for file in list(
+            itertools.chain.from_iterable(
+                self.shared.builder.plugins.applyiterhook("Tree.prune_before", tree)
+                for tree in self.walk(dirs=True, files=True)
             )
+        ):
+            self.prune(file)
 
         # Compile files
         with ThreadPoolExecutor(
@@ -564,27 +619,23 @@ class Root(Directory):
             for path in self.walk(dirs=False, files=True):
                 pool.submit(path.compile)
 
-        # Remove files that were not compiled (no plugin found to compile them)
-        for path in self.walk(dirs=False, files=True):
-            if path.report is None:
-                self.prune(path.from_source)
-
-        # Remove dependencies
-        for path in list(self.walk(dirs=False, files=True)):
-            if not self.find(path.from_source):
-                # `path` may have been removed during the loop
-                continue
-            for file in path.iter_depends():
-                self.prune(file)
+        # Prune files that are no longer needed.
+        for file in list(
+            itertools.chain.from_iterable(
+                self.shared.builder.plugins.applyiterhook("Tree.prune_after", tree)
+                for tree in self.walk(dirs=True, files=True)
+            )
+        ):
+            self.prune(file)
 
         # "Compile" directories
         for path in reversed(list(self.walk(dirs=True, files=False))):
             path.compile()
 
     @classmethod
-    def from_vcs(cls, repository):
+    def from_vcs(cls, repository: plugins.vcs.VCS) -> Root:
         """Return a directory, fully set."""
-        with cls(repository.source, vcs=repository, shared=repository.shared) as tree:
-            for path in repository.walk():
-                tree.add_subpath(pathlib.Path(path))
-            return tree
+        tree = cls(repository.source, vcs=repository, shared=repository.shared)
+        for path in repository.walk():
+            tree.add_subpath(path)
+        return tree
