@@ -86,9 +86,7 @@ class TestCompilation(unittest.TestCase):
 
         Return a set a loaded plugins.
         """
-        setup = Setup(
-            {"setup": {"vcs": "vcs.git"}, "vcs": {"enable_plugins": ["vcs.git"]}}
-        )
+        setup = Setup({"setup": {"vcs": "vcs.git"}, "vcs": {"plugins": ["vcs.git"]}})
         setup.update(local, extend_list=True)
 
         with Builder.from_setupdict(setup) as builder:
